from . import cli_entry
from . import cli_func


@cli_entry
def cli_report(args):
    return report(args)


@cli_func
def report(args, __name__="report"):
    from pathlib import Path
    from ..api import report
    from ..lib.util.run import get_module_options
    from ..lib.consts import MODULE_OPTIONS_KEY

    if MODULE_OPTIONS_KEY in args:
        module_options = get_module_options(args.get(MODULE_OPTIONS_KEY))
        args["module_options"] = module_options
    else:
        args["module_options"] = None
    if args["savepath"]:
        args["savepath"] = Path(args["savepath"])
    if args["output_dir"]:
        args["output_dir"] = Path(args["output_dir"])
    ret = report(**args)
    return ret


def get_parser_fn_report():
    from argparse import ArgumentParser, SUPPRESS
    from ..lib.system.consts import DEFAULT_SERVER_DEFAULT_USERNAME

    parser_ov_report = ArgumentParser(
        prog="ov report dbpath ...",
        description="Generate reports from result SQLite files",
        epilog="dbpath must be the first argument.",
    )
    parser_ov_report.add_argument("dbpath", help="Path to aggregator output")
    parser_ov_report.add_argument(
        "-t",
        dest="report_types",
        nargs="*",
        default=None,
        help="report types",
    )
    parser_ov_report.add_argument(
        "-f", dest="filterpath", default=None, help="Path to filter file"
    )
    parser_ov_report.add_argument("--filter", default=None, help=SUPPRESS)
    parser_ov_report.add_argument("--filtersql", default=None, help="Filter SQL")
    parser_ov_report.add_argument(
        "-F",
        dest="filtername",
        default=None,
        help="Name of filter (stored in aggregator output)",
    )
    parser_ov_report.add_argument(
        "--filterstring", dest="filterstring", default=None, help=SUPPRESS
    )
    parser_ov_report.add_argument(
        "-s", dest="savepath", default=None, help="Path to save file"
    )
    parser_ov_report.add_argument("-c", dest="confpath", help="path to a conf file")
    parser_ov_report.add_argument(
        "--module-paths",
        dest="module_paths",
        nargs="*",
        default=None,
        help="report module name",
    )
    parser_ov_report.add_argument(
        "--nogenelevelonvariantlevel",
        dest="nogenelevelonvariantlevel",
        action="store_true",
        default=False,
        help="Use this option to prevent gene level result from being added to variant level result.",
    )
    parser_ov_report.add_argument(
        "--inputfiles",
        nargs="+",
        dest="inputfiles",
        default=None,
        help="Original input file path",
    )
    parser_ov_report.add_argument(
        "--separatesample",
        dest="separatesample",
        action="store_true",
        default=False,
        help="Write each variant-sample pair on a separate line",
    )
    parser_ov_report.add_argument(
        "-d", dest="output_dir", default=None, help="directory for output files"
    )
    parser_ov_report.add_argument(
        "--quiet",
        action="store_true",
        default=None,
        help="Suppress output to STDOUT",
    )
    parser_ov_report.add_argument(
        "--module-options",
        dest="module_options",
        nargs="*",
        help="Module-specific option in module_name.key=value syntax. For example, --module-options vcfreporter.type=separate",
    )
    parser_ov_report.add_argument(
        "--includesample",
        dest="includesample",
        nargs="+",
        default=None,
        help="Sample IDs to include",
    )
    parser_ov_report.add_argument(
        "--excludesample",
        dest="excludesample",
        nargs="+",
        default=None,
        help="Sample IDs to exclude",
    )
    parser_ov_report.add_argument(
        "--package", help="Use filters and report types in a package"
    )
    parser_ov_report.add_argument(
        "--cols",
        dest="cols",
        nargs="+",
        default=None,
        help="columns to include in reports",
    )
    parser_ov_report.add_argument(
        "--level",
        default=None,
        help="Level to make a report for. 'all' to include all levels. Other possible levels include 'variant' and 'gene'.",
    )
    parser_ov_report.add_argument(
        "--user",
        default=DEFAULT_SERVER_DEFAULT_USERNAME,
        help=f"User who is creating this report. Default is {DEFAULT_SERVER_DEFAULT_USERNAME}.",
    )
    parser_ov_report.add_argument(
        "--no-summary",
        action="store_true",
        default=False,
        help="Skip gene level summarization. This saves time.",
    )
    parser_ov_report.add_argument(
        "--logtofile",
        action="store_true",
        help="Use this option to prevent gene level result from being added to variant level result.",
    )
    parser_ov_report.set_defaults(func=cli_report)
    return parser_ov_report



from typing import Optional
from oakvar import BaseAnnotator


class Annotator(BaseAnnotator):
    def annotate(self, input_data: dict, secondary_data: Optional[dict] = None):
        assert input_data is not None
        _ = secondary_data
        out = {}
        return out



from oakvar import BasePostAggregator


class PostAggregator(BasePostAggregator):
    def annotate(self, input_data: dict) -> dict:
        """
        Returns a dict of the annotation result for an input variant.

        Parameters:
            input_data: a dict of the result by the annotation modules
                        run for the input variant. Depending on what
                        annotation modules were run, the fields of
                        this dict vary. For example,
                        base__uid, base__chrom, base__pos, ...

        Returns:
            dict: a dict of variant annotation.
        """
        assert input_data is not None
        out = {}
        return out


_ = PostAggregator



from typing import Optional
from typing import Union


def user() -> Optional[dict]:
    """Gets user configuration.

    Returns:
        OakVar user configuration
    """
    from ..lib.util.admin_util import get_user_conf

    conf = get_user_conf()
    return conf


def system(
    key: Optional[str] = None, value: Optional[str] = None, type: str = "str"
) -> Optional[Union[str, int, float, dict]]:
    """Gets or sets system configuration.

    Args:
        key (Optional[str]): key
        value (Optional[str]): value
        type (Optional[str]): type

    Returns:
        With no argument, system configuration is returned as a dict. With only key given, the system configuration value for the key is returned. With key, value, and type given, system configuration is updated with the value of the type for the key. If type is omitted, str is assumed.
    """
    from ..lib.system import get_sys_conf_value
    from ..lib.system import set_sys_conf_value
    from ..lib.system import get_system_conf

    if key:
        if value:
            if not type:
                type = "str"
            set_sys_conf_value(key, value, type)
            return value
        else:
            v = get_sys_conf_value(key)
            return v
    else:
        conf = get_system_conf()
        return conf



from typing import Optional
from pathlib import Path


def exampleinput(directory: Optional[str] = ".", outer=None) -> Optional[Path]:
    """exampleinput.

    Args:
        directory (Optional[str]): Directory to create the example input file in
        outer:

    Returns:
        `None` if the given directory does not exist. Path to the created example input file if successful.
    """
    from ..lib.util.admin_util import fn_new_exampleinput

    if not directory:
        return None
    ret = fn_new_exampleinput(directory)
    if outer:
        outer.write(ret)
    return ret


def module(module_name: str, module_type: str, outer=None) -> Optional[Path]:
    """module.

    Args:
        module_name (str): Module name
        module_type (str): Module type

    Returns:
        `None` if not successful. Directory of the created module if successful.
    """
    from ..lib.util.admin_util import create_new_module
    from ..lib.module.local import get_local_module_info
    from ..lib.exceptions import ArgumentError

    if not module_name:
        e = ArgumentError("module_name should not be empty.")
        e.traceback = False
        raise e
    if not module_type:
        e = ArgumentError("module_type should not be empty.")
        e.traceback = False
        raise e
    ret = create_new_module(module_name, module_type, outer=outer)
    if not ret:
        return
    module_info = get_local_module_info(module_name)
    if module_info is not None:
        return module_info.directory
    else:
        return None



from oakvar import BaseMapper


class Mapper(BaseMapper):
    def map(self, input_data: dict) -> dict:
        """
        Returns a dict of the result of mapping an input variant
        to a gene model.

        Parameters:
            input_data: a dict of a variant. It should have the following
            fields:
                chrom: chromosome
                pos: position
                ref_base: reference bases
                alt_base: alternate bases

        Returns:
            dict: a dict of variant mapping. The following fields are
                  mandatory.
                  chrom: str
                  pos: int
                  ref_base: str
                  alt_base: str
                  transcript: primary transcript, str
                  so: sequence ontology of the input variant on
                      the primary transcript, [str]
                  cchange: cDNA change by the input variant on
                           the primary transcript, str
                  achange: amino acid change by the input variant on
                           the primary transcript, str
                  all_mappings: list, dict
        """
        assert input_data is not None
        out = {}
        return out



from typing import Optional
from pathlib import Path


def get_system_message_db_conn():
    import sqlite3
    from .consts import SYSTEM_MESSAGE_DB_FNAME
    from .consts import SYSTEM_MESSAGE_TABLE
    from ..lib.system import get_conf_dir
    from ..lib.system import get_default_conf_dir
    from ..lib.exceptions import SystemMissingException

    conf_dir: Optional[Path] = get_conf_dir()
    if not conf_dir:
        conf_dir = get_default_conf_dir()
    if not conf_dir:
        raise SystemMissingException(
            msg="Configuration directory does not exist. Please run "
            + "`ov system setup` to setup OakVar."
        )
    if not Path(conf_dir).exists():
        conf_dir.mkdir(parents=True)
    db_path = conf_dir / SYSTEM_MESSAGE_DB_FNAME
    conn = sqlite3.connect(db_path)
    c = conn.cursor()
    c.execute(
        "select * from sqlite_master where type='table' and "
        + f"name='{SYSTEM_MESSAGE_TABLE}'"
    )
    ret = c.fetchone()
    if not ret:
        create_system_message_db(conn)
    return conn


def create_system_message_db(conn):
    from .consts import SYSTEM_MESSAGE_TABLE
    from .consts import SYSTEM_ERROR_TABLE

    c = conn.cursor()
    c.execute(f"drop table if exists {SYSTEM_MESSAGE_TABLE}")
    c.execute(
        f"create table {SYSTEM_MESSAGE_TABLE} (uid integer primary key, "
        + "kind text, msg text, dt float)"
    )
    c.execute(f"drop table if exists {SYSTEM_ERROR_TABLE}")
    c.execute(
        f"create table {SYSTEM_ERROR_TABLE} (uid integer primary key, "
        + "kind text, msg text, dt float)"
    )
    conn.commit()


def clear_system_message_db(conn):
    from .consts import SYSTEM_MESSAGE_TABLE
    from .consts import SYSTEM_ERROR_TABLE

    c = conn.cursor()
    c.execute(f"delete from {SYSTEM_MESSAGE_TABLE}")
    c.execute(f"delete from {SYSTEM_ERROR_TABLE}")
    conn.commit()


def get_last_msg_id(conn):
    from .consts import SYSTEM_MESSAGE_TABLE

    c = conn.cursor()
    c.execute(f"select max(uid) from {SYSTEM_MESSAGE_TABLE}")
    ret = c.fetchone()
    if ret and ret[0]:
        return ret[0]
    else:
        return 0



from . import cli_entry
from . import cli_func


@cli_entry
def cli_util_test(args):
    return test(args)


@cli_func
def test(args, __name__="util test"):
    from ..api.test import test

    ret = test(**args)
    if ret:
        num_failed: int = ret.get("num_failed", 0)
        if num_failed > 0:
            return False
        else:
            return True
    return True


def get_parser_cli_util_test():
    from argparse import ArgumentParser

    parser_cli_util_test = ArgumentParser()
    parser_cli_util_test.add_argument("-d", "--rundir", help="Directory for output")
    parser_cli_util_test.add_argument(
        "-m", "--modules", nargs="+", help="Name of module(s) to test. (e.g. gnomad)"
    )
    parser_cli_util_test.add_argument(
        "--quiet", action="store_true", default=None, help="run quietly"
    )
    parser_cli_util_test.set_defaults(func=cli_util_test)
    return parser_cli_util_test


def main():
    args = get_parser_cli_util_test().parse_args()
    cli_util_test(args)


if __name__ == "__main__":
    main()



from typing import List
from typing import Dict
from oakvar import BaseConverter


class Converter(BaseConverter):
    def check_format(self, f) -> bool:
        """
        Detect the format of an input file.

        Arguments:
            f: a file handle to an input file
        Returns:
            bool: True if the input file is for this converter,
                  False if not.

        The example below checks if the input file's first line indicates
        VCF file format.
        """
        line = f.readline()
        return line.startswith("##fileformat=VCF")

    # If your converter module needs something else than
    # the standard way of opening a text input file,
    # read line by line, and coverting each line into
    # a list of dictionaries of variants,
    # you may want to start with modifying
    # convert_file method. In that case, uncomment
    # the below convert_file method and add your implementation.
    #
    # def convert_file(
    #     self, file, *__args__, exc_handler=None, **__kwargs__
    # ) -> Iterator[Tuple[int, List[dict]]]:
    #     line_no = 0
    #     for line in file:
    #         line_no += 1
    #         try:
    #             yield line_no, self.convert_line(line)
    #         except Exception as e:
    #             if exc_handler:
    #                 exc_handler(line_no, e)
    #             else:
    #                 raise e
    #     return None

    def convert_line(self, line) -> List[Dict]:
        """
        Converts a line from an input file to OakVar's variant dict.

        Arguments:
            l: a string of a line from an input file
        Returns:
            dict: a list of dicts, each dict for a variant collected
                  from the input line. Each dict should have
                  the following required fields:

                  chrom: chromosome name [str]
                  pos: chromosomal position [int]
                  ref_base: reference bases [str]
                  alt_base: altername bases [str]

                  Optional fields for each dict are:

                  sample_id: the ID or name of a sample having the variant [list[str]]
                  tags: a custom tag given to the variant [list[str]]
        """
        _ = line
        var_dicts = []
        var_dict = {
            "chrom": "chr1",
            "pos": 2878349,
            "ref_base": "A",
            "alt_base": "T",
            "sample_id": "sample1",
        }
        var_dicts.append(var_dict)
        return var_dicts



from typing import Optional
from typing import Union
from typing import Dict
from pathlib import Path


def setup(
    clean: bool = False,
    refresh_db: bool = False,
    clean_cache_files: bool = False,
    setup_file: Optional[Union[Path, str]] = None,
    email: Optional[str] = None,
    pw: Optional[str] = None,
    custom_system_conf: Optional[Dict] = None,
    publish_time: str = "",
    outer=None,
    system_worker_state=None,
):
    """setup.

    Args:
        clean (bool): Perform clean installation. Installed modules and analysis results are not erased.
        refresh_db (bool): Refreshes store server data.
        clean_cache_files (bool): Cleans store cache files.
        setup_file (Optional[Union[Path, str]]): Path to a custom system configuration file. If given, the system configuraton from this file will be used instead of default system configuratoin values.
        email (Optional[str]): OakVar store account Email
        pw (Optional[str]): OakVar store account password
        custom_system_conf (Optional[Dict]): Custom system configuration as a Dict
        publish_time (str): publish_time
        system_worker_state:
        outer:
    """
    from ..lib.system import setup_system

    if isinstance(setup_file, str):
        setup_file = Path(setup_file)
    return setup_system(
        clean=clean,
        refresh_db=refresh_db,
        clean_cache_files=clean_cache_files,
        setup_file=setup_file,
        email=email,
        pw=pw,
        publish_time=publish_time,
        custom_system_conf=custom_system_conf,
        outer=outer,
        system_worker_state=system_worker_state,
    )


def md(directory: Optional[Union[Path, str]] = None) -> Optional[Path]:
    """Gets or sets OakVar modules directory.

    Args:
        directory (Optional[Union[Path, str]]): Path to a new OakVar modules directory. If given, OakVar modules directory will be set to this value.

    Returns:
        Path of the new or existing OakVar modules directory. `None` if `directory` is not given and an OakVar modules directory is not defined in the system configuration.
    """
    from ..lib.system import set_modules_dir, get_modules_dir

    if directory:
        set_modules_dir(directory)
        return Path(directory)
    else:
        d = get_modules_dir()
        return d


def check(outer=None) -> bool:
    """Performs OakVar system checkup.

    Args:
        outer:

    Returns:
        True if all tests passed. False if not.
    """
    from ..lib.system import check

    ret = check(outer=outer)
    return ret



from typing import Optional


def create(
    email: Optional[str] = None,
    pw: Optional[str] = None,
    pwconfirm: bool = False,
    interactive: bool = False,
    outer=None,
) -> bool:
    """Creates an OakVar store account.

    Args:
        email (Optional[str]): Email of an OakVar store account
        pw (Optional[str]): Password of an OakVar store account
        pwconfirm (bool): Should be the same as `pw`.
        interactive (bool): If `True` and `email` or `pw` is not given, missing fields will be interactvely received with prompts.
        outer:

    Returns:
        `True` if successful. `False` if not.
    """
    from ....lib.store.ov.account import create

    ret = create(
        email=email, pw=pw, pwconfirm=pwconfirm, interactive=interactive, outer=outer
    )
    success: bool = ret.get("success", False)
    return success


def delete(outer=None) -> bool:
    """Deletes an OakVar store account. You should be already logged in.

    Args:
        outer:

    Returns:
        `True` if successful. `False` if not.
    """
    from ....lib.store.ov.account import delete

    ret = delete(outer=outer)
    return ret


def change(newpw: Optional[str] = None, outer=None) -> bool:
    """Changes the password of an OakVar store account. You should be already logged in.

    Args:
        newpw (Optional[str]): New password
        outer:

    Returns:
        `True` if successful. `False` if not.
    """
    from ....lib.store.ov.account import change

    ret = change(newpw=newpw, outer=outer)
    return ret


def reset(email: Optional[str], outer=None) -> bool:
    """Sends a password reset email for an OakVar store account. You should be already logged in.

    Args:
        email (Optional[str]): Email of the logged in OakVar store account
        outer:

    Returns:
        `True` if successful. `False` if not.
    """
    from ....lib.store.ov.account import reset

    ret = reset(email=email, outer=outer)
    return ret


def check(outer=None) -> bool:
    """Checks if you are logged in to the OakVar store.

    Args:
        outer:

    Returns:
        `True` if logged in. `False` if not.
    """
    from ....lib.store.ov.account import check_logged_in_with_token

    ret = check_logged_in_with_token(outer=outer)
    return ret



from oakvar import BasePreparer


class Preparer(BasePreparer):
    def prepare(self, input_data: dict):
        self.writer.write_data(input_data)

    def postloop(self):
        pass


_ = Preparer



from . import admin_util
from . import asyn
from . import download_library
from . import download
from . import image
from . import inout
from . import run
from . import seq
from . import util
from .util import get_ucsc_bins

_ = (
    admin_util
    or asyn
    or download_library
    or download
    or image
    or inout
    or run
    or seq
    or util
    or get_ucsc_bins
)



from . import cli_entry
from . import cli_func


@cli_entry
def cli_new_exampleinput(args):
    exampleinput(args)


@cli_func
def exampleinput(args, __name__="new exampleinput"):
    from ..api.new import exampleinput

    return exampleinput(args.get("directory"))


@cli_entry
def cli_new_module(args):
    return module(args)


@cli_func
def module(args, __name__="new module"):
    from ..api.new import module

    ret = module(args.get("name"), args.get("type"), outer=args.get("outer"))
    if ret:
        print(f"Created {ret}")


def get_parser_fn_new():
    from argparse import ArgumentParser, RawDescriptionHelpFormatter

    parser_fn_new = ArgumentParser(formatter_class=RawDescriptionHelpFormatter)
    _subparsers = parser_fn_new.add_subparsers(title="Commands")

    # test input file
    parser_cli_new_exampleinput = _subparsers.add_parser(
        "exampleinput", help="makes a file with example input variants."
    )
    parser_cli_new_exampleinput.add_argument(
        "-d",
        dest="directory",
        default=".",
        help="Directory to make the example input file in",
    )
    parser_cli_new_exampleinput.add_argument(
        "--quiet", action="store_true", default=None, help="run quietly"
    )
    parser_cli_new_exampleinput.set_defaults(func=cli_new_exampleinput)
    parser_cli_new_exampleinput.r_return = "A string. Location of the example input file"  # type: ignore
    parser_cli_new_exampleinput.r_examples = [  # type: ignore
        "# Create an example input file in the current working directory",
        "#roakvar::new.exampleinput()",
        "# Create an example input file at /home/user1/",
        '#roakvar::new.exampleinput(directory="/home/user1")',
    ]

    # new-annotator
    parser_cli_new_annotator = _subparsers.add_parser(
        "module", help="creates a new module skeleton."
    )
    parser_cli_new_annotator.add_argument("-n", dest="name", help="Module name")
    parser_cli_new_annotator.add_argument("-t", dest="type", help="Module type")
    parser_cli_new_annotator.add_argument(
        "--quiet", action="store_true", default=None, help="No print to stdout"
    )
    parser_cli_new_annotator.set_defaults(func=cli_new_module)
    parser_cli_new_annotator.r_return = "A string. Location of the new annotator module"  # type: ignore
    parser_cli_new_annotator.r_examples = [  # type: ignore
        "# Create an annotator template at the OakVar modules directory/annotators/annotatortest",
        '#roakvar::new.annotator(annotator_name="annotatortest")',
    ]
    return parser_fn_new



from . import cli_entry
from . import cli_func


@cli_entry
def cli_config_user(args):
    return user(args)


@cli_func
def user(args, __name__="config user"):
    import oyaml as yaml
    from ..api.config import user

    conf = user()
    conf = yaml.dump(conf, default_flow_style=False)
    outer = args.get("outer", None)
    if outer:
        outer.write(conf)


@cli_entry
def cli_config_system(args):
    return system(args)


@cli_func
def system(args, __name__="config system"):
    from rich.console import Console
    from rich.table import Table
    from rich.box import SQUARE
    from ..api.config import system

    key = args.get("key")
    value = args.get("value")
    ty = args.get("type")
    ret = system(key=key, value=value, type=ty)
    if isinstance(ret, str):
        print(ret)
    elif isinstance(ret, dict):
        console = Console()
        table = Table(title=ret.get("sys_conf_path"), box=SQUARE)
        table.add_column("Key")
        table.add_column("Value")
        for k, v in ret.items():
            table.add_row(k, str(v))
        console.print(table)


def get_parser_fn_config():
    from argparse import ArgumentParser, RawDescriptionHelpFormatter

    parser_fn_config = ArgumentParser(formatter_class=RawDescriptionHelpFormatter)
    subparsers = parser_fn_config.add_subparsers(title="Commands")
    add_parser_ov_config_user(subparsers)
    add_parser_ov_config_system(subparsers)
    return parser_fn_config


def add_parser_ov_config_user(subparsers):
    # shows oakvar conf content.
    parser_cli_config_oakvar = subparsers.add_parser(
        "user",
        epilog="A dictionary. content of OakVar user configuration file",
        help="shows oakvar user configuration",
    )
    parser_cli_config_oakvar.set_defaults(func=cli_config_user)
    parser_cli_config_oakvar.r_return = "A named list. OakVar user config information"  # type: ignore
    parser_cli_config_oakvar.r_examples = [  # type: ignore
        "# Get the named list of the OakVar user configuration",
        "#roakvar::config.user()",
    ]


def add_parser_ov_config_system(subparsers):
    # shows oakvar conf content.
    parser_cli_config_oakvar = subparsers.add_parser(
        "system",
        epilog="A dictionary. content of OakVar system configuration file",
        help="shows oakvar system configuration",
    )
    parser_cli_config_oakvar.add_argument("key", nargs="?", help="Configuration key")
    parser_cli_config_oakvar.add_argument(
        "value", nargs="?", help="Configuration value"
    )
    parser_cli_config_oakvar.add_argument(
        "type", nargs="?", help="Type of configuration value"
    )
    parser_cli_config_oakvar.add_argument(
        "--fmt", default="json", help="Format of output: table / json"
    )
    parser_cli_config_oakvar.set_defaults(func=cli_config_system)
    parser_cli_config_oakvar.r_return = "A named list. OakVar system config information"  # type: ignore
    parser_cli_config_oakvar.r_examples = [  # type: ignore
        "# Get the named list of the OakVar system configuration",
        "#roakvar::config.system()",
    ]



from . import lib
from . import api
from . import cli
from .lib import consts
from .lib.base.runner import Runner
from .lib.base.converter import BaseConverter
from .lib.base.master_converter import MasterConverter
from .lib.base.preparer import BasePreparer
from .lib.base.mapper import BaseMapper
from .lib.base.annotator import BaseAnnotator
from .lib.base.postaggregator import BasePostAggregator
from .lib.base.report_filter import ReportFilter
from .lib.base.reporter import BaseReporter
from .lib.base.commonmodule import BaseCommonModule
from .lib.base.vcf2vcf import VCF2VCF
from .lib.util.inout import FileReader
from .lib.util.inout import FileWriter
from .lib.util import inout
from .lib.util import admin_util
from .lib.util.util import get_df_from_db
from .lib.util.inout import read_crv
from .lib.util.seq import get_lifter
from .lib.util.seq import liftover
from .lib.util.seq import get_wgs_reader
from .cli import CliOuter
import signal

# for compatibility with oc
from .lib.exceptions import BadFormatError
from .lib.exceptions import InvalidData

Cravat = Runner
CravatReport = BaseReporter
BaseReport = BaseReporter
CravatFilter = ReportFilter
constants = consts

#

stdouter = CliOuter()


def raise_break(__signal_number__, __stack_frame__):
    import os
    import platform
    import psutil

    pl = platform.platform()
    if pl.startswith("Windows"):
        pid = os.getpid()
        for child in psutil.Process(pid).children(recursive=True):
            try:
                child.kill()
            except psutil.NoSuchProcess:  # keep
                pass
        os.kill(pid, signal.SIGTERM)
    elif pl.startswith("Linux"):
        pid = os.getpid()
        for child in psutil.Process(pid).children(recursive=True):
            try:
                child.kill()
            except psutil.NoSuchProcess:  # keep
                pass
        os.kill(pid, signal.SIGTERM)
    elif pl.startswith("Darwin") or pl.startswith("macOS"):
        pid = os.getpid()
        for child in psutil.Process(pid).children(recursive=True):
            try:
                child.kill()
            except psutil.NoSuchProcess:  # keep
                pass
        os.kill(pid, signal.SIGTERM)


def get_annotator(module_name, input_file=None):

    module = None
    input_file = input_file or "__dummy__"
    ModuleClass = get_module(module_name)
    if ModuleClass:
        module = ModuleClass(input_file=input_file)
        module.connect_db()
        module.setup()
    return module


def get_mapper(module_name, input_file=None):
    module = None
    ModuleClass = get_module(module_name, module_type="mapper")
    if ModuleClass:
        module = ModuleClass(input_file=input_file)
        module.name = module_name
        module.setup()
    return module


def get_converter(module_name):
    module = None
    ModuleClass = get_module(module_name, module_type="converter")
    if ModuleClass:
        module = ModuleClass()
        module.name = module_name
        module.setup()
    return module


def get_module(module_name, module_type: str = ""):
    from os.path import dirname
    from .lib.module.local import get_local_module_info
    from .lib.module.local import get_module_conf
    from .lib.util.util import load_class

    ModuleClass = None
    module_conf = get_module_conf(module_name, module_type=module_type)
    module_info = get_local_module_info(module_name)
    if module_info is not None:
        script_path = module_info.script_path
        ModuleClass = load_class(script_path)
        ModuleClass.script_path = script_path
        ModuleClass.module_name = module_name
        ModuleClass.module_dir = dirname(script_path)
        ModuleClass.conf = module_conf
    return ModuleClass


wgs = None
_ = api or lib
_ = BadFormatError or InvalidData
_ = admin_util or inout
_ = (
    BaseConverter
    or MasterConverter
    or BasePreparer
    or BaseAnnotator
    or BaseMapper
    or BasePostAggregator
    or BaseCommonModule
    or VCF2VCF
    or CravatReport
    or ReportFilter
    or Runner
    or FileReader
    or FileWriter
)
_ = CravatFilter or Cravat
_ = cli or wgs
_ = stdouter
_ = get_lifter or liftover or get_wgs_reader
_ = get_df_from_db or read_crv
